/**
 * @function doCreateSign
 */
'use strict'

const co = require('co')
const doCreate = require('apeman-app-rest/lib/doing/do_create')
const uuid = require('uuid')

/** @lends doCreateSign */
function doCreateSign (config) {
  let { digest } = config

  let _dataBeforeCreate = (data) => co(function * () {
    let attributes = data.attributes
    let password = attributes.password
    attributes.hashSalt = uuid.v4()
    attributes.hash = yield digest(password, attributes.hashSalt)
    attributes.signupAt = new Date()
    delete attributes.password
    return Object.assign(data, { attributes })
  })

  let _dataAfterCreate = (data) => co(function * () {
    let attributes = data.attributes
    delete attributes.hash
    delete attributes.hashSalt
    return Object.assign(data, {
      attributes
    })
  })

  function createSign (UserSign, context) {
    let isBulk = Array.isArray(context.data)
    if (isBulk) {
      return createSign.bulk(UserSign, context)
    }
    return co(function * () {
      if (context.data) {
        context.data = yield _dataBeforeCreate(context.data)
      }
      let created = yield doCreate(UserSign, context)
      if (created.data) {
        created.data = yield _dataAfterCreate(created.data)
      }
      return created
    })
  }

  Object.assign(createSign, {
    bulk (UserSign, context) {
      return co(function * () {
        if (context.data) {
          context.data = yield [].concat(context.data).map(_dataBeforeCreate)
        }
        let created = yield doCreate.bulk(UserSign, context)
        if (created.data) {
          created.data = yield [].concat(created.data).map(_dataAfterCreate)
        }
        return created
      })
    }
  })

  return createSign
}

module.exports = doCreateSign
