/**
 * @function doOneSignPass
 */
'use strict'

const co = require('co')
const errors = require('../errors')
const apemanmodel = require('apemanmodel')
const successPayload = require('apeman-app-rest/lib/payloads/success_payload')
const failurePayload = require('apeman-app-rest/lib/payloads/failure_payload')
const ApSignPassModel = require('../models').ApSignPassModel

/** @lends doOneSignPass */
function doOneSignPass (config) {
  const ACTION_NAME = 'one'
  let success = successPayload(ACTION_NAME)
  let fail = failurePayload(ACTION_NAME)
  let type = apemanmodel.toType(ApSignPassModel)

  let _includedAfterOne = (included) => co(function * () {
    for (let i of [].concat(included || [])) {
      let attributes = i.attributes
      delete attributes.hash
      delete attributes.hashSalt
    }
    return included
  })

  function oneSignPass (UserSign, context) {
    let id = context.id
    let oneSign = apemanmodel.defineOne(UserSign, context.transaction)
    return co(function * () {
      let req = context.req
      let signed = req.signed()
      let hit = signed && signed.id === id
      if (!hit) {
        return fail(errors.notFoundError(type, id))
      }
      let sign = yield oneSign(signed.sign.id)
      let result = success(
        {},
        {
          type,
          id,
          relationships: {
            sign: {
              data: apemanmodel.toData.withoutAttributes(UserSign, sign)
            }
          }
        },
        {},
        [
          apemanmodel.toData(UserSign, sign, {})
        ]
      )
      result.included = yield _includedAfterOne(result.included)
      return result
    })
  }

  return oneSignPass
}

module.exports = doOneSignPass
